package config

import (
	"github.com/zeromicro/go-zero/core/logx"
	"github.com/zeromicro/go-zero/zrpc"
)

type (
	Kafka struct {
		Addr  []string // 连接地址
		Topic string
	}

	// Config 配置
	Config struct {
		zrpc.RpcServerConf
		Log logx.LogConf
		DB  struct {
			DataSource string
		}
		// Kafka 消息队列配置
		Kafka struct {
			UserNotify Kafka // 用户信息通知队列
			RankUpdate Kafka // 排行榜更新通知队列
		}
		UserRetriever struct {
			Enabled           bool     // 是否开启
			UpdateDuration    int64    // 用户信息更新最短间隔 单位 h
			NobilityPlatforms []string // 需要获取贵族的平台
			Cron              struct {
				PlatformUser string
				Nobility     string
			}
			Bilibili struct {
				RoomId           int64  // 直播房间号
				Mid              int64  `json:",optional"` // 主播UID 不输入就通过API获取
				RoomShortInfoApi string // 房间简短信息API
				TopListApi       string
			}
		}
		Rank struct {
			Enabled bool // 是否开启
			Cron    struct {
				Update      string // 更新榜单
				Persistence string // 持久化
			}
		}
		GiftCollector struct {
			Enabled   bool     // 是否开启
			Platforms []string // 需搜集的平台
			Cron      struct {
				CollectGift string
			}
		}
		Integral struct {
			// CheckIn 签到
			CheckIn struct {
				Points        []int64   // 签到积分 次数分数
				Critical      float32   // 暴击率(百分比)
				CriticalRadio []float32 // 暴击倍数
			}
			// 战局积分
			BattleReport struct {
				GeneralIntegral int64   // 名将积分
				WinRadio        float64 // 获胜方积分因子 乘法
				LostRadio       float64 // 失败方积分因子 乘法
			}
			RMBToIntegral  float64            // RMB到积分的转换
			GiftToRMB      map[string]float64 // 平台礼物到RMB的转换
			FreeToIntegral map[string]float64 // 平台免费礼物到积分的转换
		}
	}
)
