package statistics

import (
	"context"
	"github.com/pkg/errors"
	"gorm.io/gorm"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"
	"live-service/common/nerr"

	"github.com/zeromicro/go-zero/core/logx"
)

type StatPvpFirstBloodLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewStatPvpFirstBloodLogic(ctx context.Context, svcCtx *svc.ServiceContext) *StatPvpFirstBloodLogic {
	return &StatPvpFirstBloodLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

const (
	TypeFirstBlood   = 1
	TypeDeFirstBlood = 2
)

func (l *StatPvpFirstBloodLogic) StatPvpFirstBlood(in *pb.StatPvPFirstBloodReq) (*pb.Empty, error) {
	// 虚拟UID
	if in.Uid <= 0 {
		return &pb.Empty{}, nil
	}
	err := l.svcCtx.StatisticsPvpModel.Transaction(l.ctx, nil, func(tx *gorm.DB) error {
		// 直接更新，提高效率
		props := model.UpdateRecordProps{}
		if in.Type == TypeFirstBlood {
			props.FirstBlood = true
		} else if in.Type == TypeDeFirstBlood {
			props.DeFirstBlood = true
		}

		if err := l.svcCtx.StatisticsPvpModel.UpdateRecord(l.ctx, tx, in.Uid,
			&props); err != nil {
			if errors.Is(err, model.ErrRowsAffectedZero) {
				// insert
				dbModel := model.StatisticsPvp{
					UserId: in.Uid,
				}
				if in.Type == TypeFirstBlood {
					dbModel.FirstBloodCount = 1
				} else if in.Type == TypeDeFirstBlood {
					dbModel.DeFirstBloodCount = 1
				}
				if err = l.svcCtx.StatisticsPvpModel.InsertTx(l.ctx, tx, &dbModel); err != nil {
					return errors.Wrapf(err, ErrInsertErr, err)
				}
			} else {
				return errors.Wrapf(err, ErrUpdateErr, err)
			}
		}
		return nil
	})
	if err != nil {
		return nil, errors.Wrapf(nerr.NewWithCode(nerr.DBError), "一血报送-事务执行失败, err:%+v", err)
	}
	return &pb.Empty{}, nil
}
