package statistics

import (
	"context"
	"github.com/pkg/errors"
	"gorm.io/gorm"
	"live-service/app/user_center/model"
	"live-service/common/nerr"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type StatPvpKillLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewStatPvpKillLogic(ctx context.Context, svcCtx *svc.ServiceContext) *StatPvpKillLogic {
	return &StatPvpKillLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *StatPvpKillLogic) StatPvpKill(in *pb.StatPvPKillReq) (*pb.Empty, error) {
	// 虚拟UID
	if in.Uid <= 0 {
		return &pb.Empty{}, nil
	}

	err := l.svcCtx.StatisticsPvpModel.Transaction(l.ctx, nil, func(tx *gorm.DB) error {
		// 击杀
		props := model.UpdateRecordProps{
			KillPlayer: true,
		}
		if err := l.svcCtx.StatisticsPvpModel.UpdateRecord(l.ctx, tx, in.Uid, &props); err != nil {
			if errors.Is(err, model.ErrRowsAffectedZero) {
				dbModel := &model.StatisticsPvp{
					UserId:          in.Uid,
					KillPlayerCount: 1,
				}
				if err := l.svcCtx.StatisticsPvpModel.InsertTx(l.ctx, tx, dbModel); err != nil {
					return errors.Wrapf(err, ErrInsertErr, err)
				}
			} else {
				return errors.Wrapf(err, ErrUpdateErr, err)
			}
		}
		// 被杀 + 落马否
		// 虚拟UID
		if in.TargetUid <= 0 {
			return nil
		}
		props = model.UpdateRecordProps{
			DeKillPlayer: true,
			DeGeneral:    in.IsGeneral,
		}
		if err := l.svcCtx.StatisticsPvpModel.UpdateRecord(l.ctx, tx, in.TargetUid, &props); err != nil {
			if errors.Is(err, model.ErrRowsAffectedZero) {
				dbModel := &model.StatisticsPvp{
					UserId:          in.TargetUid,
					DeKillUnitCount: 1,
				}
				if in.IsGeneral {
					dbModel.DeGeneralCount = 1
				}
				if err := l.svcCtx.StatisticsPvpModel.InsertTx(l.ctx, tx, dbModel); err != nil {
					return errors.Wrapf(err, ErrInsertErr, err)
				}
			} else {
				return errors.Wrapf(err, ErrUpdateErr, err)
			}
		}
		return nil
	})
	if err != nil {
		return nil, errors.Wrapf(nerr.NewWithCode(nerr.DBError), "击杀玩家-事务执行失败, err:%+v", err)
	}
	return &pb.Empty{}, nil
}
